Package: graphics
Version: 4.1.0
Priority: base
Title: The R Graphics Package
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <do-use-Contact-address@r-project.org>
Contact: R-help mailing list <r-help@r-project.org>
Description: R functions for base graphics.
Imports: grDevices
License: Part of R 4.1.0
NeedsCompilation: yes
Built: R 4.1.0; x86_64-w64-mingw32; 2021-05-21 10:09:36 UTC; windows
