Package: compiler
Version: 4.1.0
Priority: base
Title: The R Compiler Package
Author: Luke Tierney <luke-tierney@uiowa.edu>
Maintainer: R Core Team <do-use-Contact-address@r-project.org>
Contact: R-help mailing list <r-help@r-project.org>
Description: Byte code compiler for R.
License: Part of R 4.1.0
Built: R 4.1.0; ; 2021-05-21 10:08:18 UTC; windows
