Package: grid
Version: 4.0.5
Priority: base
Title: The Grid Graphics Package
Author: Paul Murrell <paul@stat.auckland.ac.nz>
Maintainer: R Core Team <R-core@r-project.org>
Description: A rewrite of the graphics layout capabilities, plus some
  support for interaction.
Imports: grDevices, utils
Suggests: lattice
License: Part of R 4.0.5
NeedsCompilation: yes
Built: R 4.0.5; x86_64-w64-mingw32; 2021-05-14 18:02:37 UTC; windows
