Package: tcltk
Version: 4.1.0
Priority: base
Title: Tcl/Tk Interface
Author: R Core Team
Maintainer: R Core Team <do-use-Contact-address@r-project.org>
Contact: R-help mailing list <r-help@r-project.org>
Description: Interface and language bindings to Tcl/Tk GUI elements.
License: Part of R 4.1.0
Imports: utils
NeedsCompilation: yes
Built: R 4.1.0; x86_64-w64-mingw32; 2021-05-21 10:11:20 UTC; windows
