Package: stats
Version: 4.1.0
Priority: base
Title: The R Stats Package
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <do-use-Contact-address@r-project.org>
Contact: R-help mailing list <r-help@r-project.org>
Description: R statistical functions.
License: Part of R 4.1.0
Imports: utils, grDevices, graphics
Suggests: MASS, Matrix, SuppDists, methods, stats4
NeedsCompilation: yes
Built: R 4.1.0; x86_64-w64-mingw32; 2021-05-21 10:09:50 UTC; windows
