Package: grDevices
Version: 4.0.5
Priority: base
Title: The R Graphics Devices and Support for Colours and Fonts
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <R-core@r-project.org>
Description: Graphics devices and support for base and grid graphics.
License: Part of R 4.0.5
Suggests: KernSmooth
NeedsCompilation: yes
Built: R 4.0.5; x86_64-w64-mingw32; 2021-05-14 18:00:50 UTC; windows
