Package: parallel
Version: 3.5.1
Priority: base
Title: Support for Parallel computation in R
Author: R Core Team
Maintainer: R Core Team <R-core@r-project.org>
Description: Support for parallel computation, including by forking
   (taken from package multicore), by sockets (taken from package snow)
   and random-number generation.
License: Part of R 3.5.1
Imports: tools, compiler
Suggests: methods
Enhances: snow, nws, Rmpi
NeedsCompilation: yes
Built: R 3.5.1; x86_64-w64-mingw32; 2019-07-02 00:40:16 UTC; windows
