Package: methods
Version: 4.0.5
Priority: base
Imports: utils, stats
Title: Formal Methods and Classes
Author: R Core Team
Maintainer: R Core Team <R-core@r-project.org>
Description: Formally defined methods and classes for R objects,
  plus other programming tools, as described in the reference.
References: John M. Chambers (2008) ``Software for Data Analysis:
        Programming with R''; Springer NY.
License: Part of R 4.0.5
Suggests: codetools
NeedsCompilation: yes
Built: R 4.0.5; x86_64-w64-mingw32; 2021-05-14 18:02:15 UTC; windows
