Package: utils
Version: 3.5.1
Priority: base
Title: The R Utils Package
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <R-core@r-project.org>
Description: R utility functions.
License: Part of R 3.5.1
Suggests: methods, xml2, commonmark
NeedsCompilation: yes
Built: R 3.5.1; x86_64-w64-mingw32; 2019-07-02 00:39:57 UTC; windows
