Package: tools
Version: 4.0.5
Priority: base
Title: Tools for Package Development
Author: R Core Team
Maintainer: R Core Team <R-core@r-project.org>
Description: Tools for package development, administration and documentation.
License: Part of R 4.0.5
Suggests: codetools, methods, xml2, curl, commonmark
NeedsCompilation: yes
Built: R 4.0.5; x86_64-w64-mingw32; 2021-05-14 17:59:43 UTC; windows
