Package: datasets
Version: 4.0.5
Priority: base
Title: The R Datasets Package
Author: R Core Team and contributors worldwide
Maintainer: R Core Team <R-core@r-project.org>
Description: Base R datasets.
License: Part of R 4.0.5
Built: R 4.0.5; ; 2021-05-14 18:02:12 UTC; windows
