bibentry(bibtype = "Article",
         key = "R:Snedecor:1948",
         .issn = "0006341X, 15410420",
         doi = "10.2307/3001566",
         journal = "Biometrics",
         number = "3",
         pages = "213--215",
         publisher = "[Wiley, International Biometric Society]",
         author = person(given = c("George", "W."),
                         family = "Snedecor"),
         title = "60. Query",
         volume = "4",
         year = "1948")

bibentry(bibtype = "Article",
         key = "R:Anscombe:1973",
         title = "Graphs in Statistical Analysis",
         volume = "27",
         .issn = "1537-2731",
         doi = "10.1080/00031305.1973.10478966",
         number = "1",
         journal = "The American Statistician",
         publisher = "Informa UK Limited",
         author = person(given = c("Francis", "John"),
                         family = "Anscombe"),
         year = "1973",
         month = "feb",
         pages = "17--21",
         .doi = "10.2307/2682899")

bibentry(bibtype = "Article",
         key = "R:Azzalini+Bowman:1990",
         title = "A Look at Some Data on the {Old Faithful} Geyser",
         volume = "39",
         .issn = "0035-9254",
         doi = "10.2307/2347385",
         number = "3",
         journal = "Applied Statistics",
         publisher = "JSTOR",
         author = c(person(given = "Adelchi",
                           family = "Azzalini"),
                    person(given = c("Adrian", "W."),
                           family = "Bowman",
                           comment = c(ORCID = "0000-0002-5729-4631"))),
         year = "1990",
         pages = "357")

bibentry(bibtype = "Article",
         key = "R:Balke:1993",
         title = "Detecting Level Shifts in Time Series",
         volume = "11",
        .issn = "1537-2707",
         doi = "10.1080/07350015.1993.10509934",
         number = "1",
         journal = "Journal of Business &amp; Economic Statistics",
         publisher = "Informa UK Limited",
         author = person(given = c("Nathan", "S."),
                         family = "Balke"),
         year = "1993",
         month = "jan",
         pages = "81--92",
         .doi = "10.2307/1391308")

bibentry(bibtype = "Book",
         key = "R:Bates+Watts:1988",
         title = "Nonlinear Regression Analysis and Its Applications",
         author = c(person(given = c("Douglas", "M."),
                           family = "Bates"),
                    person(given = c("Donald", "G."),
                           family = "Watts")),
         isbn = "9780471816430",
         lccn = "88006065",
         series = "Wiley Series in Probability and Statistics",
         year = "1988",
         publisher = "Wiley")

bibentry(bibtype = "Article",
         key = "R:Bickel+Hammel+O_Connell:1975",
         title = "Sex Bias in Graduate Admissions: Data from Berkeley: Measuring bias is harder than is usually assumed, and the evidence is sometimes contrary to expectation.",
         volume = "187",
         .issn = "1095-9203",
         doi = "10.1126/science.187.4175.398",
         number = "4175",
         journal = "Science",
         publisher = "American Association for the Advancement of Science (AAAS)",
         author = c(person(given = c("Peter", "John"),
                           family = "Bickel"),
                    person(given = c("Eugene", "A."),
                           family = "Hammel"),
                    person(given = c("J.", "W."),
                           family = "O'Connell")),
         year = "1975",
         month = "feb",
         pages = "398--404")

bibentry(bibtype = "Manual",
         key = "R:Boeckmann+Sheiner+Beal:1994",
         title = "{NONMEM} Users Guide: Part V",
         author = c(person(given = c("A.", "J."),
                           family = "Boeckmann"),
                    person(given = c("Lewis", "B."),
                           family = "Sheiner"),
                    person(given = c("Stuart", "L."),
                           family = "Beal")),
         organization = "NONMEM Project Group, University of California",
         address = "San Francisco",
         year = "1994")

bibentry(bibtype = "Article",
         key = "R:Bolt+Abrahamson:1982",
         author = c(person(given = c("Bruce", "A."),
                           family = "Bolt"),
                    person(given = c("Norman", "A."),
                           family = "Abrahamson")),
         title = "New Attenuation Relations for Peak and Expected Accelerations of Strong Ground Motion",
         journal = "Bulletin of the Seismological Society of America",
         volume = "72",
         number = "6A",
         pages = "2307--2322",
         year = "1982",
         month = "12",
         .issn = "0037-1106",
         doi = "10.1785/BSSA07206A2307",
         .eprint = "https://pubs.geoscienceworld.org/ssa/bssa/article-pdf/72/6A/2307/5330679/bssa07206a2307.pdf")

bibentry(bibtype = "Article",
         key = "R:Bolt+Abrahamson:1983",
         author = c(person(given = c("Bruce", "A."),
                           family = "Bolt"),
                    person(given = c("Norman", "A."),
                           family = "Abrahamson")),
         title = "Reply to {W. B. Joyner} and {D. M. Boore}'s “Comments on ‘New Attenuation Relations for Peak and Expected Accelerations of Strong Ground Motion’”",
         journal = "Bulletin of the Seismological Society of America",
         volume = "73",
         number = "5",
         pages = "1481--1483",
         year = "1983",
         month = "10",
         .issn = "0037-1106",
         doi = "10.1785/BSSA0730051481",
         .eprint = "https://pubs.geoscienceworld.org/ssa/bssa/article-pdf/73/5/1481/5331782/bssa0730051481.pdf")

bibentry(bibtype = "Article",
         key = "R:Boore+Joyner:1982",
         author = c(person(given = c("David", "M."),
                           family = "Boore"),
                    person(given = c("William", "B."),
                           family = "Joyner")),
         title = "The Empirical Prediction of Ground Motion",
         journal = "Bulletin of the Seismological Society of America",
         volume = "72",
         number = "6B",
         pages = "S43--S60",
         year = "1982",
         month = "12",
         .issn = "0037-1106",
         doi = "10.1785/BSSA07206B0043",
         .eprint = "https://pubs.geoscienceworld.org/ssa/bssa/article-pdf/72/6B/S43/5330804/bssa07206b0043.pdf")

bibentry(bibtype = "Article",
         key = "R:Brillinger+Preisler:1984",
         author = c(person(given = c("David", "R."),
                           family = "Brillinger"),
                    person(given = c("Haiganoush", "K."),
                           family = "Preisler")),
         title = "An Exploratory Analysis of the {Joyner}-{Boore} Attenuation Data",
         journal = "Bulletin of the Seismological Society of America",
         volume = "74",
         number = "4",
         pages = "1441--1450",
         year = "1984",
         month = "08",
         .issn = "0037-1106",
         doi = "10.1785/BSSA0740041441",
         .eprint = "https://pubs.geoscienceworld.org/ssa/bssa/article-pdf/74/4/1441/5332305/bssa0740041441.pdf")

bibentry(bibtype = "Article",
         key = "R:Brillinger+Preisler:1985",
         author = c(person(given = c("David", "R."),
                           family = "Brillinger"),
                    person(given = c("Haiganoush", "K."),
                           family = "Preisler")),
         title = "Further Analysis of the {Joyner}-{Boore} Attenuation Data",
         journal = "Bulletin of the Seismological Society of America",
         volume = "75",
         number = "2",
         pages = "611--614",
         year = "1985",
         doi = "10.1785/BSSA0750020611")

bibentry(bibtype = "Article",
         key = "R:Campbell+Walker:1977",
         title = "A Survey of Statistical Work on the Mackenzie River Series of Annual Canadian Lynx Trappings for the Years 1821-1934 and a New Analysis",
         volume = "140",
         .issn = "0035-9238",
         doi = "10.2307/2345277",
         number = "4",
         journal = "Journal of the Royal Statistical Society. Series A (General)",
         publisher = "JSTOR",
         author = c(person(given = c("Mike", "J."),
                           family = "Campbell"),
                    person(given = c("A.", "M."),
                           family = "Walker")),
         year = "1977",
         pages = "411")

bibentry(bibtype = "Article",
         key = "R:Clette+Lefevre+Chatzistergos:2023",
         title = "Recalibration of the Sunspot-Number: Status Report",
         volume = "298",
         .issn = "1573-093X",
         doi = "10.1007/s11207-023-02136-3",
         number = "3",
         journal = "Solar Physics",
         publisher = "Springer Science and Business Media LLC",
         author = c(person(given = "Frédéric",
                           family = "Clette",
                           comment = c(ORCID = "0000-0002-3343-5153")),
                    person(given = "Laure",
                           family = "Lefèvre",
                           comment = c(ORCID = "0000-0003-1005-7353")),
                    person(given = "Theodosios",
                           family = "Chatzistergos",
                           comment = c(ORCID = "0000-0002-0335-9831")),
                    person(given = "HHisashi",
                           family = "Hayakawa",
                           comment = c(ORCID = "0000-0001-5370-3365")),
                    person(given = c("Víctor", "Manuel", "Sánchez"),
                           family = "Carrasco",
                           comment = c(ORCID = "0000-0001-9358-1219")),
                    person(given = "Rainer",
                           family = "Arlt"),
                    person(given = c("Edward", "W."),
                           family = "Cliver",
                           comment = c(ORCID = "0000-0002-4342-6728")),
                    person(given = "Thierry",
                           family = "Dudok de Wit",
                           comment = c(ORCID = "0000-0002-4401-0943")),
                    person(given = c("Thomas", "K."),
                           family = "Friedli",
                           comment = c(ORCID = "0000-0002-2086-7192")),
                    person(given = "Nina",
                           family = "Karachik"),
                    person(given = "Greg",
                           family = "Kopp"),
                    person(given = "Mike",
                           family = "Lockwood",
                           comment = c(ORCID = "0000-0002-7397-2172")),
                    person(given = "Sophie",
                           family = "Mathieu",
                           comment = c(ORCID = "0000-0003-2105-9733")),
                    person(given = "Andrés",
                           family = "Muñoz-Jaramillo",
                           comment = c(ORCID = "0000-0002-4716-0840")),
                    person(given = "Mathew",
                           family = "Owens",
                           comment = c(ORCID = "0000-0003-2061-2453")),
                    person(given = "Dean",
                           family = "Pesnell",
                           comment = c(ORCID = "0000-0002-8306-2500")),
                    person(given = "Alexei",
                           family = "Pevtsov",
                           comment = c(ORCID = "0000-0003-0489-0920")),
                    person(given = "Leif",
                           family = "Svalgaard",
                           comment = c(ORCID = "0000-0002-7375-7448")),
                    person(given = c("Ilya", "G."),
                           family = "Usoskin",
                           comment = c(ORCID = "0000-0001-8227-9081")),
                    person(given = "Lidia",
                           family = "van Driel-Gesztelyi",
                           comment = c(ORCID = "0000-0002-2943-5978")),
                    person(given = c("José", "Manuel"),
                           family = "Vaquero",
                           comment = c(ORCID = "0000-0002-8754-1509"))),
         year = "2023",
         month = "mar")

bibentry(bibtype = "Article",
         key = "R:Clette+Lefevre:2016",
         title = "The New Sunspot Number: Assembling All Corrections",
         volume = "291",
         .issn = "1573-093X",
         doi = "10.1007/s11207-016-1014-y",
         number = "9--10",
         journal = "Solar Physics",
         publisher = "Springer Science and Business Media LLC",
         author = c(person(given = "Frédéric",
                           family = "Clette",
                           comment = c(ORCID = "0000-0002-3343-5153")),
                    person(given = "Laure",
                           family = "Lefèvre",
                           comment = c(ORCID = "0000-0003-1005-7353"))),
         year = "2016",
         month = "nov",
         pages = "2629--2651")

bibentry(bibtype = "Article",
         key = "R:Cobb:1978",
         title = "The Problem of the {Nile}: Conditional Solution to a Changepoint Problem",
         volume = "65",
         .issn = "1464-3510",
         doi = "10.1093/biomet/65.2.243",
         number = "2",
         journal = "Biometrika",
         publisher = "Oxford University Press (OUP)",
         author = person(given = c("George", "W."),
                         family = "Cobb"),
         year = "1978",
         month = "aug",
         pages = "243--251",
         .doi = "10.2307/2335202")

bibentry(bibtype = "Article",
         key = "R:Crampton:1947",
         title = "The Growth of the Odontoblasts of the Incisor Tooth as a Criterion of the Vitamin {C} Intake of the Guinea Pig",
         volume = "33",
         .issn = "0022-3166",
         doi = "10.1093/jn/33.5.491",
         number = "5",
         journal = "The Journal of Nutrition",
         publisher = "Elsevier BV",
         author = person(given = c("Earle", "Wilcox"),
                         family = "Crampton"),
         year = "1947",
         month = "may",
         pages = "491--504")

bibentry(bibtype = "Book",
         key = "R:Davidian+Giltinan:1995",
         title = "Nonlinear Models for Repeated Measurement Data",
         author = c(person(given = "Marie",
                           family = "Davidian"),
                    person(given = c("David", "M."),
                           family = "Giltinan")),
         isbn = "9780412983412",
         lccn = "gb95055534",
         series = "Chapman & Hall/CRC Monographs on Statistics & Applied Probability",
         year = "1995",
         publisher = "Taylor & Francis")

bibentry(bibtype = "Article",
         key = "R:Dawson:1995",
         title = "The “Unusual Episode” Data Revisited",
         volume = "3",
         .issn = "1069-1898",
         doi = "10.1080/10691898.1995.11910499",
         number = "3",
         journal = "Journal of Statistics Education",
         publisher = "Informa UK Limited",
         author = person(given = c("Robert", "J.", "MacG."),
                         family = "Dawson"),
         year = "1995",
         month = "nov")

bibentry(bibtype = "Article",
         key = "R:Fisher:1936",
         title = "The Use of Multiple Measurements in Taxonomic Problems",
         volume = "7",
         .issn = "2050-1439",
         doi = "10.1111/j.1469-1809.1936.tb02137.x",
         number = "2",
         journal = "Annals of Eugenics",
         publisher = "Wiley",
         author = person(given = c("Ronald", "A."),
                         family = "Fisher"),
         year = "1936",
         month = "sep",
         pages = "179--188")

bibentry(bibtype = "InProceedings",
         key = "R:Friendly:1992b",
         author = person(given = "Michael",
                         family = "Friendly"),
         title = "Mosaic Displays for Loglinear Models",
         booktitle = "Proceedings of the Statistical Graphics Section",
         year = "1992",
         pages = "61--68",
         organization = "American Statistical Association",
         url = "http://www.datavis.ca/papers/asa92.html")

bibentry(bibtype = "Book",
         key = "R:Friendly:2000",
         title = "Visualizing Categorical Data",
         author = person(given = "Michael",
                         family = "Friendly", 
                         comment = c(ORCID = "0000-0002-3237-0941")),
         isbn = "9781580256605",
         lccn = "2001270863",
         series = "SAS series in statistical applications",
         year = "2000",
         publisher = "SAS Institute")

bibentry(bibtype = "Article",
         key = "R:Garson:1900",
         title = "The Metric System of Identification of Criminals, as Used in {Great Britain} and {Ireland}",
         volume = "30",
         .issn = "0959-5295",
         doi = "10.2307/2842627",
         journal = "The Journal of the Anthropological Institute of Great Britain and Ireland",
         publisher = "JSTOR",
         author = person(given = c("John", "George"),
                         family = "Garson"),
         year = "1900",
         pages = "161")

bibentry(bibtype = "Article",
         key = "R:Gorman+Williams+Fraser:2014",
         title = "Ecological Sexual Dimorphism and Environmental Variability within a Community of Antarctic Penguins (Genus Pygoscelis)",
         volume = "9",
         .issn = "1932-6203",
         doi = "10.1371/journal.pone.0090081",
         number = "3",
         journal = "PLoS ONE",
         publisher = "Public Library of Science (PLoS)",
         author = c(person(given = c("Kristen", "B."),
                           family = "Gorman",
                           comment = c(ORCID = "0000-0002-0258-9264")),
                    person(given = c("Tony", "D."),
                           family = "Williams"),
                    person(given = c("William", "R."),
                           family = "Fraser")),
         editor = person(given = "André",
                         family = "Chiaradia"),
         year = "2014",
         month = "mar",
         pages = "e90081")

bibentry(bibtype = "Article",
         key = "R:Harvey+Durbin:1986",
         title = "The Effects of Seat Belt Legislation on {British} Road Casualties: A Case Study in Structural Time Series Modelling",
         volume = "149",
         .issn = "0035-9238",
         doi = "10.2307/2981553",
         number = "3",
         journal = "Journal of the Royal Statistical Society. Series A (General)",
         publisher = "JSTOR",
         author = c(person(given = c("Andrew", "C."),
                           family = "Harvey"),
                    person(given = "James",
                           family = "Durbin")),
         year = "1986",
         pages = "187")

bibentry(bibtype = "Article",
         key = "R:Henderson+Velleman:1981",
         author = c(person(given = c("Harold", "V."),
                           family = "Henderson"),
                    person(given = c("Paul", "F."),
                           family = "Velleman")),
         title = "Building Multiple Regression Models Interactively",
         journal = "Biometrics",
         year = "1981",
         volume = "37",
         number = "2",
         pages = "391--411",
         month = "jun",
         doi = "10.2307/2530428")

bibentry(bibtype = "Article",
         key = "R:Horst+Presmanes_Hill+Gorman:2022",
         title = "Palmer Archipelago Penguins Data in the palmerpenguins R Package - An Alternative to Anderson’s Irises",
         volume = "14",
         .issn = "2073-4859",
         doi = "10.32614/RJ-2022-020",
         number = "1",
         journal = "The R Journal",
         publisher = "The R Foundation",
         author = c(person(given = c("Allison", "M."),
                           family = "Horst",
                           comment = c(ORCID = "0000-0002-6047-5564")),
                    person(given = "Alison",
                           family = "Presmanes Hill",
                           comment = c(ORCID = "0000-0002-8082-1890")),
                    person(given = c("Kristen", "B."),
                           family = "Gorman",
                           comment = c(ORCID = "0000-0002-0258-9264"))),
         year = "2022",
         month = "jun",
         pages = "244--254",
         .doi = "10.32614/rj-2022-020")

bibentry(bibtype = "Book",
         key = "R:Haerdle:1991",
         title = "Smoothing Techniques: With Implementation in {S}",
         author = person(given = c("Wolfgang", "Karl"),
                         family = "Härdle",
                         comment = c(ORCID = "0000-0001-5600-3014")),
         isbn = "9780387973678",
         lccn = "90047545",
         series = "Springer Series in Statistics",
         year = "1991",
         publisher = "Springer")

bibentry(bibtype = "Misc",
         key = "R:Kaye+Turner+Gorman:2025",
         doi = "10.5281/zenodo.14902740",
         author = c(person(given = "Ella",
                           family = "Kaye",
                           comment = c(ORCID = "0000-0002-7300-3718")),
                    person(given = "Heather",
                           family = "Turner",
                           comment = c(ORCID = "0000-0002-1256-3375")),
                    person(given = c("Kristen", "B."),
                           family = "Gorman",
                           comment = c(ORCID = "0000-0002-0258-9264")),
                    person(given = c("Allison", "M."),
                           family = "Horst",
                           comment = c(ORCID = "0000-0002-6047-5564")),
                    person(given = "Alison",
                           family = "Presmanes Hill",
                           comment = c(ORCID = "0000-0002-8082-1890"))),
         title = "Preparing the Palmer Penguins Data for the datasets Package in R",
         publisher = "Zenodo",
         year = "2025",
         copyright = "Creative Commons Attribution 4.0 International",
         .doi = "10.5281/ZENODO.14902740")

bibentry(bibtype = "Article",
         key = "R:MacDonell:1902",
         title = "On Criminal Anthropometry and the Identification of Criminals",
         volume = "1",
         .issn = "1464-3510",
         doi = "10.1093/biomet/1.2.177",
         number = "2",
         journal = "Biometrika",
         publisher = "Oxford University Press (OUP)",
         author = person(given = c("W.", "R."),
                         family = "MacDonell"),
         year = "1902",
         month = "jan",
         pages = "177--227",
         .doi = "10.2307/2331487")

bibentry(bibtype = "Article",
         key = "R:Michelson:1882",
         author = person(given = c("Albert", "Abraham"),
                         family = "Michelson"),
         title = "Experimental Determination of the Velocity of Light: Made at the {U.S. Naval Academy}, {Annapolis}",
         journal = "Astronomical Papers",
         year = "1882",
         month = "jan",
         volume = "1",
         pages = "109--145",
         publisher = "United States Nautical Almanac Office",
         url = "https://ui.adsabs.harvard.edu/abs/1882USNAO...1..109M",
         note = "Provided by the SAO/NASA Astrophysics Data System")

bibentry(bibtype = "Article",
         key = "R:Olshen+Biden+Wyatt:1989",
         title = "Gait Analysis and the Bootstrap",
         volume = "17",
         .issn = "0090-5364",
         doi = "10.1214/aos/1176347372",
         number = "4",
         journal = "The Annals of Statistics",
         publisher = "Institute of Mathematical Statistics",
         author = c(person(given = c("Richard", "A."),
                           family = "Olshen"),
                    person(given = c("Edmund", "N."),
                           family = "Biden"),
                    person(given = c("Marilynn", "P."),
                           family = "Wyatt"),
                    person(given = c("David", "H."),
                           family = "Sutherland")),
         year = "1989",
         month = "dec")

bibentry(bibtype = "Misc",
         key = "R:Palmer_Station_Antarctica_LTER+Gorman:2020a",
         doi = "10.6073/pasta/98b16d7d563f265cb52372c8ca99e60f",
         author = c(person(given = "Palmer Station Antarctica LTER"),
                    person(given = c("Kristen", "B."),
                           family = "Gorman",
                           comment = c(ORCID = "0000-0002-0258-9264"))),
         language = "en",
         title = "Structural size measurements and isotopic signatures of foraging among adult male and female Adélie penguins (Pygoscelis adeliae) nesting along the Palmer Archipelago near Palmer Station, 2007-2009",
         publisher = "Environmental Data Initiative",
         year = "2020",
         .doi = "10.6073/PASTA/98B16D7D563F265CB52372C8CA99E60F")

bibentry(bibtype = "Misc",
         key = "R:Palmer_Station_Antarctica_LTER+Gorman:2020b",
         doi = "10.6073/pasta/7fca67fb28d56ee2ffa3d9370ebda689",
         author = c(person(given = "Palmer Station Antarctica LTER"),
                    person(given = c("Kristen", "B."),
                           family = "Gorman",
                           comment = c(ORCID = "0000-0002-0258-9264"))),
         language = "en",
         title = "Structural Size Measurements and Isotopic Signatures of Foraging among Adult Male and Female {Gentoo} Penguin ({Pygoscelis papua}) Nesting along the {Palmer Archipelago} near {Palmer Station}, 2007-2009",
         publisher = "Environmental Data Initiative",
         year = "2020",
         .doi = "10.6073/PASTA/7FCA67FB28D56EE2FFA3D9370EBDA689")

bibentry(bibtype = "Misc",
         key = "R:Palmer_Station_Antarctica_LTER+Gorman:2020c",
         doi = "10.6073/pasta/c14dfcfada8ea13a17536e73eb6fbe9e",
         author = c(person(given = "Palmer Station Antarctica LTER"),
                    person(given = c("Kristen", "B."),
                           family = "Gorman",
                           comment = c(ORCID = "0000-0002-0258-9264"))),
         language = "en",
         title =
         "Structural Size Measurements and Isotopic Signatures of Foraging among Adult Male and Female {Chinstrap} Penguin ({Pygoscelis antarctica}) Nesting along the {Palmer Archipelago} near {Palmer Station}, 2007-2009",
         publisher = "Environmental Data Initiative",
         year = "2020",
         .doi = "10.6073/PASTA/C14DFCFADA8EA13A17536E73EB6FBE9E")

bibentry(bibtype = "Book",
         key = "R:Pinheiro+Bates:2000",
         title = "Mixed-Effects Models in S and S-PLUS",
         author = c(person(given = "José",
                           family = "Pinheiro"),
                    person(given = c("Douglas", "M."),
                           family = "Bates",
                           comment = c(ORCID = "0000-0001-8316-9503"))),
         isbn = "9780387989570",
         lccn = "99053566",
         series = "Statistics and Computing",
         year = "2000",
         publisher = "Springer New York")

bibentry(bibtype = "Book",
         key = "R:Ramsay+Silverman:2005",
         title = "Functional Data Analysis",
         author = c(person(given = "James",
                           family = "Ramsay"),
                    person(given = c("Bernard", "Walter"),
                           family = "Silverman")),
         isbn = "9780387400808",
         lccn = "2005923773",
         series = "Springer Series in Statistics",
         year = "2005",
         publisher = "Springer")

bibentry(bibtype = "Manual",
         key = "R:Ramsay:2023",
         title = "{fda}: Functional Data Analysis",
         author = person(given = "James",
                         family = "Ramsay"),
         year = "2023",
         note = "R package version 6.1.4",
         url = "https://CRAN.R-project.org/package=fda",
         doi = "10.32614/CRAN.package.fda")

bibentry(bibtype = "Manual",
         key = "R:Ramsay:2025",
         title = "fda: Functional Data Analysis",
         author = person(given = "James",
                         family = "Ramsay"),
         year = "2025",
         note = "R package version 6.3.0",
         url = "https://CRAN.R-project.org/package=fda",
         doi = "10.32614/CRAN.package.fda")

bibentry(bibtype = "Article",
         key = "R:Smith+Stanley:1983",
         title = "Clocking g: Relating Intelligence and Measures of Timed Performance",
         volume = "7",
         .issn = "0160-2896",
         doi = "10.1016/0160-2896(83)90010-7",
         number = "4",
         journal = "Intelligence",
         publisher = "Elsevier BV",
         author = c(person(given = c("Glen", "A."),
                           family = "Smith"),
                    person(given = "Gordon",
                           family = "Stanley")),
         year = "1983",
         month = "oct",
         pages = "353--368")

bibentry(bibtype = "Article",
         key = "R:Snee:1974",
         title = "Graphical Display of Two-way Contingency Tables",
         volume = "28",
         .issn = "1537-2731",
         doi = "10.1080/00031305.1974.10479053",
         number = "1",
         journal = "The American Statistician",
         publisher = "Informa UK Limited",
         author = person(given = c("Ronald", "D."),
                         family = "Snee"),
         year = "1974",
         month = "feb",
         pages = "9--12",
         .doi = "10.2307/2683520")

bibentry(bibtype = "Article",
         key = "R:Student:1908",
         title = "The Probable Error of a Mean",
         volume = "6",
         .issn = "0006-3444",
         doi = "10.2307/2331554",
         number = "1",
         journal = "Biometrika",
         publisher = "JSTOR",
         author = person(given = "Student"),
         year = "1908",
         month = "mar",
         pages = "1")

bibentry(bibtype = "MastersThesis",
         key = "R:Treloar:1974",
         author = person(given = c("Margaret", "A."),
                         family = "Treloar"),
         title = "Effects of Puromycin on Galactosyltransferase in Golgi Membranes",
         school = "University of Toronto",
         year = "1974")

bibentry(bibtype = "Article",
         key = "R:Trichopoulos+Handanos+Danezis:1976",
         title = "Induced Abortion and Secondary Infertility",
         volume = "83",
         .issn = "1471-0528",
         doi = "10.1111/j.1471-0528.1976.tb00904.x",
         number = "8",
         journal = "BJOG: An International Journal of Obstetrics & Gynaecology",
         publisher = "Wiley",
         author = c(person(given = "D.",
                           family = "Trichopoulos"),
                    person(given = "N.",
                           family = "Handanos"),
                    person(given = "J.",
                           family = "Danezis"),
                    person(given = "Anna",
                           family = "Kalandidi"),
                    person(given = "Victoria",
                           family = "Kalapothaki")),
         year = "1976",
         month = "aug",
         pages = "645--650")

bibentry(bibtype = "Book",
         key = "R:Atkinson:1986",
         author = person(given = c("Anthony", "C."),
                         family = "Atkinson"),
         title = "Plots, Transformations and Regression",
         publisher = "Oxford University Press",
         year = "1986",
         isbn = "978-0198533597")

bibentry(bibtype = "Book",
         key = "R:Box+Jenkins:1976",
         title = "Time Series Analysis: Forecasting and Control",
         author = c(person(given = c("George", "Edward", "Pelham"),
                           family = "Box"),
                    person(given = c("Gwilym", "M."),
                           family = "Jenkins")),
         isbn = "9780816211043",
         lccn = "lc76008713",
         series = "Holden-Day series in time series analysis and digital processing",
         year = "1976",
         publisher = "Holden-Day")

bibentry(bibtype = "Book",
         key = "R:Brown:1963",
         author = person(given = c("Robert", "Goodell"),
                         family = "Brown"),
         title = "Smoothing, Forecasting and Prediction of Discrete Time Series",
         publisher = "Prentice-Hall",
         year = "1963",
         isbn = "978-0138153083")

bibentry(bibtype = "Book",
         key = "R:Cleveland:1993",
         author = person(given = c("William", "S."),
                         family = "Cleveland"),
         title = "Visualizing Data",
         publisher = "Hobart Press",
         year = "1993",
         isbn = "978-0963488404")

bibentry(bibtype = "Book",
         key = "R:Makridakis+Wheelwright+Hyndman:1997",
         author = c(person(given = c("Spyros", "G."),
                           family = "Makridakis"),
                    person(given = c("Steven", "C."),
                           family = "Wheelwright"),
                    person(given = c("Rob", "J."),
                           family = "Hyndman")),
         title = "Forecasting: Methods and Applications",
         publisher = "John Wiley & Sons",
         year = "1997",
         isbn = "978-0471532330")

bibentry(bibtype = "Book",
         key = "R:McNeil:1977",
         author = person(given = "Don",
                         family = "McNeil"),
         title = "Interactive Data Analysis: A Practical Primer",
         publisher = "John Wiley & Sons",
         year = "1977",
         isbn = "978-0471026310")

bibentry(bibtype = "Unpublished",
         key = "R:Sterling:1977",
         author = person(given = "Arnie",
                         family = "Sterling"),
         title = "Unpublished BS Thesis",
         note = "Massachusetts Institute of Technology",
         year = "1977")

bibentry(bibtype = "InCollection",
         key = "R:Dodge:1996",
         author = person(given = "Yadolah",
                         family = "Dodge"),
         title = "The Guinea Pig of Multiple Regression",
         booktitle = "Robust Statistics, Data Analysis, and Computer Intensive Methods: In Honor of Peter Huber's 60th Birthday",
         publisher = "Springer",
         year = "1996",
         editor = person(given = "Helmut",
                         family = "Rieder"),
         volume = "109",
         series = "Lecture Notes in Statistics",
         pages = "91--117",
         address = "New York, NY",
         doi = "10.1007/978-1-4612-2380-1_7")

bibentry(bibtype = "Book",
         key = "R:Scheffe:1959",
         author = person(given = "Henry",
                         family = "Scheffé"),
         title = "The Analysis of Variance",
         publisher = "John Wiley & Sons",
         year = "1959",
         isbn = "978-0471758341")

bibentry(bibtype = "Article",
         key = "R:Kwan+Breault+Umbenhauer:1976",
         title = "Kinetics of Indomethacin Absorption, Elimination, and Enterohepatic Circulation in Man",
         volume = "4",
         .issn = "0090-466X",
         doi = "10.1007/bf01063617",
         number = "3",
         journal = "Journal of Pharmacokinetics and Biopharmaceutics",
         publisher = "Springer Science and Business Media LLC",
         author = c(person(given = c("K.", "C."),
                           family = "Kwan"),
                    person(given = c("G.", "O."),
                           family = "Breault"),
                    person(given = c("E.", "R."),
                           family = "Umbenhauer"),
                    person(given = c("F.", "G."),
                           family = "McMahon"),
                    person(given = c("D.", "E."),
                           family = "Duggan")),
         year = "1976",
         month = "jun",
         pages = "255--280")

bibentry(bibtype = "Article",
         key = "R:Potvin+Lechowicz+Tardif:1990",
         title = "The Statistical Analysis of Ecophysiological Response Curves Obtained from Experiments Involving Repeated Measures",
         volume = "71",
         .issn = "1939-9170",
         doi = "10.2307/1938276",
         number = "4",
         journal = "Ecology",
         publisher = "Wiley",
         author = c(person(given = "Catherine",
                           family = "Potvin"),
                    person(given = c("Martin", "J."),
                           family = "Lechowicz"),
                    person(given = "Serge",
                           family = "Tardif")),
         year = "1990",
         month = "aug",
         pages = "1389--1400")

bibentry(bibtype = "Article",
         key = "R:Longley:1967",
         title = "An Appraisal of Least Squares Programs for the Electronic Computer from the Point of View of the User",
         volume = "62",
         .issn = "1537-274X",
         doi = "10.1080/01621459.1967.10500896",
         number = "319",
         journal = "Journal of the American Statistical Association",
         publisher = "Informa UK Limited",
         author = person(given = c("James", "W."),
                         family = "Longley"),
         year = "1967",
         month = "sep",
         pages = "819--841")

bibentry(bibtype = "Article",
         key = "R:Beall:1942",
         title = "The Transformation of Data from Entomological Field Experiments so that the Analysis of Variance Becomes Applicable",
         volume = "32",
         .issn = "1464-3510",
         doi = "10.1093/biomet/32.3-4.243",
         number = "3--4",
         journal = "Biometrika",
         publisher = "Oxford University Press (OUP)",
         author = person(given = "Geoffrey",
                         family = "Beall"),
         year = "1942",
         pages = "243--262")

bibentry(bibtype = "Article",
         key = "R:Molyneaux+Gilliam+Florant:1947",
         title = "Differences in {Virginia} Death Rates by Color, Sex, Age and Rural or Urban Residence",
         volume = "12",
         .issn = "0003-1224",
         doi = "10.2307/2086734",
         number = "5",
         journal = "American Sociological Review",
         publisher = "SAGE Publications",
         author = c(person(given = "Lambert",
                           family = "Molyneaux"),
                    person(given = c("Sara", "K."),
                           family = "Gilliam"),
                    person(given = c("L.", "C."),
                           family = "Florant")),
         year = "1947",
         month = "oct",
         pages = "525")

bibentry(bibtype = "Article",
         key = "R:Cushny+Peebles:1905",
         title = "The Action of Optical Isomers: II. Hyoscines",
         volume = "32",
         .issn = "1469-7793",
         doi = "10.1113/jphysiol.1905.sp001097",
         number = "5--6",
         journal = "The Journal of Physiology",
         publisher = "Wiley",
         author = c(person(given = c("Arthur", "R."),
                           family = "Cushny"),
                    person(given = c("A.", "Roy"),
                           family = "Peebles")),
         year = "1905",
         month = "jul",
         pages = "501--510")

bibentry(bibtype = "InProceedings",
         key = "R:Kung:1986",
         author = person(given = c("F.", "H."),
                         family = "Kung"),
         title = "Fitting Logistic Growth Curve with Predetermined Carrying Capacity",
         booktitle = "Proceedings of the Statistical Computing Section",
         year = "1986",
         pages = "340--343",
         organization = "American Statistical Association")

bibentry(bibtype = "Article",
         key = "R:Anderson:1935",
         author = person(given = "Edgar",
                         family = "Anderson"),
         title = "The Irises of the {Gaspe} Peninsula",
         journal = "Bulletin of the American Iris Society",
         year = "1935",
         volume = "59",
         pages = "2--5")

bibentry(bibtype = "Article",
         key = "R:Stigler:1977",
         title = "Do Robust Estimators Work with Real Data?",
         volume = "5",
         .issn = "0090-5364",
         doi = "10.1214/aos/1176343997",
         number = "6",
         pages = "1055--1098",
         journal = "The Annals of Statistics",
         publisher = "Institute of Mathematical Statistics",
         author = person(given = c("Stephen", "M."),
                         family = "Stigler"),
         year = "1977",
         month = "nov")

bibentry(bibtype = "Article",
         key = "R:Goodman:1979",
         title = "Simple Models for the Analysis of Association in Cross-Classifications Having Ordered Categories",
         volume = "74",
         .issn = "1537-274X",
         doi = "10.1080/01621459.1979.10481650",
         number = "367",
         journal = "Journal of the American Statistical Association",
         publisher = "Informa UK Limited",
         author = person(given = c("Leo", "A."),
                         family = "Goodman"),
         year = "1979",
         month = "sep",
         pages = "537--552")

bibentry(bibtype = "Article",
         key = "R:Vaux+Brinker:1972",
         author = c(person(given = c("James", "E."),
                           family = "Vaux"),
                    person(given = c("Nancy", "B."),
                           family = "Brinker")),
         title = "Cotton Acreage Harvested, {U.S.A.}: The 12-Year Cycle",
         journal = "Cycles",
         year = "1972",
         pages = "6--7",
         url = "https://cycles.org/cycles/library/")

bibentry(bibtype = "Book",
         key = "R:Harman:1976",
         author = person(given = c("Harry", "Horace"),
                         family = "Harman"),
         title = "Modern Factor Analysis",
         publisher = "University of Chicago Press",
         year = "1976",
         address = "Chicago",
         edition = "Third",
         isbn = "9780226316529")

bibentry(bibtype = "Book",
         key = "R:Bartholomew:1987",
         author = person(given = c("David", "J."),
                         family = "Bartholomew"),
         title = "Latent Variable Models and Factor Analysis",
         publisher = "Griffin",
         year = "1987",
         isbn = "9780195206159")

bibentry(bibtype = "Book",
         key = "R:Box+Jenkins+Reinsel:1994",
         author = c(person(given = c("George", "Edward", "Pelham"),
                           family = "Box"),
                    person(given = c("Gwilym", "M."),
                           family = "Jenkins"),
                    person(given = c("Gregory", "C."),
                           family = "Reinsel")),
         title = "Time Series Analysis: Forecasting and Control",
         edition = "Third",
         isbn = "978-0130607744",
         year = "1994",
         publisher = "Prentice Hall",
         pointers = "https://permalink.obvsg.at/AC01651302, https://search.worldcat.org/de/title/28888762")

bibentry(bibtype = "InCollection",
         key = "R:Reynolds:1994",
         author = person(given = c("P.", "S."),
                         family = "Reynolds"),
         title = "Time-series Analyses of Beaver Body Temperatures",
         booktitle = "Case Studies in Biometry",
         publisher = "Wiley",
         year = "1994",
         editor = person(given = "Nicholas",
                         family = "Lange"),
         chapter = "11",
         address = "New York")

bibentry(bibtype = "MastersThesis",
         key = "R:Marske:1967",
         author = person(given = c("D.", "M."),
                         family = "Marske"),
         title = "Biochemical Oxygen Demand Data Interpretation Using Sum of Squares Surface",
         school = "University of Wisconsin -- Madison",
         year = "1967")

bibentry(bibtype = "Book",
         key = "R:Crowder+Hand:1990",
         author = c(person(given = c("Martin", "J."),
                           family = "Crowder"),
                    person(given = c("David", "J."),
                           family = "Hand")),
         title = "Analysis of Repeated Measures",
         publisher = "Chapman & Hall",
         year = "1990",
         address = "London",
         isbn = "041231830X",
         pointers = "https://permalink.obvsg.at/AC00242159")

bibentry(bibtype = "Book",
         key = "R:Hand+Crowder:1996",
         author = c(person(given = c("David", "J."),
                           family = "Hand"),
                    person(given = c("Martin", "J."),
                           family = "Crowder")),
         title = "Practical Longitudinal Data Analysis",
         publisher = "Chapman & Hall",
         year = "1996",
         address = "London",
         isbn = "0412599406",
         pointers = "https://permalink.obvsg.at/AC01274978")

bibentry(bibtype = "Book",
         key = "R:Bennett+Franklin:1954",
         author = c(person(given = c("Carl", "A."),
                           family = "Bennett"),
                    person(given = c("Norman", "L."),
                           family = "Franklin")),
         title = "Statistical Analysis in Chemistry and the Chemical Industry",
         publisher = "Wiley",
         year = "1954",
         address = "New York, NY",
         isbn = "9780471066330",
         pointers = "https://permalink.obvsg.at/AC01178416")

bibentry(bibtype = "Book",
         key = "R:Shumway+Stoffer:2000",
         author = c(person(given = c("Robert", "H."),
                           family = "Shumway"),
                    person(given = c("David", "S."),
                           family = "Stoffer")),
         title = "Time Series Analysis and its Applications",
         publisher = "Springer",
         year = "2000",
         address = "New York, NY",
         isbn = "0387989501",
         pointers = "https://permalink.obvsg.at/AC02922027")

bibentry(bibtype = "Book",
         key = "R:Draper+Smith:1998",
         author = c(person(given = c("Norman", "Richard"),
                           family = "Draper"),
                    person(given = "Harry",
                           family = "Smith")),
         title = "Applied Regression Analysis",
         publisher = "Wiley",
         year = "1998",
         address = "New York, NY",
         edition = "Third",
         isbn = "9780471170822",
         pointers = "https://permalink.obvsg.at/AC02409562")

bibentry(bibtype = "Book",
         key = "R:Finney:1947",
         author = person(given = c("David", "John"),
                         family = "Finney"),
         title = "Probit Analysis",
         publisher = "Cambridge University Press",
         year = "1947",
         address = "Cambridge")

bibentry(bibtype = "Book",
         key = "R:British_Government:1990",
         author = person(given = "British Government"),
         title = "Report on the Loss of the {SS Titanic}",
         publisher = "Allan Sutton Publishing",
         year = "1990",
         address = "Gloucester, UK",
         isbn = "978-0862997236")

bibentry(bibtype = "Book",
         key = "R:Bliss:1952",
         author = person(given = c("Chester", "Ittner"),
                         family = "Bliss"),
         title = "The Statistics of Bioassay",
         publisher = "Academic Press",
         year = "1952",
         address = "New York, NY",
         isbn = "9781483256627",
         pointers = "https://permalink.obvsg.at/AC01773949")

bibentry(bibtype = "Book",
         key = "R:Diggle:1991",
         author = person(given = c("Peter", "J."),
                         family = "Diggle"),
         title = "Time Series: A Biostatistical Introduction",
         publisher = "Clarendon Press",
         year = "1991",
         address = "Oxford",
         isbn = "0198522266",
         pointers = "https://permalink.obvsg.at/AC00531463")

bibentry(bibtype = "Book",
         key = "R:Bartholomew+Knott:1999",
         author = c(person(given = c("David", "J."),
                           family = "Bartholomew"),
                    person(given = "Martin",
                           family = "Knott")),
         title = "Latent Variable Analysis and Factor Analysis",
         publisher = "Arnold",
         year = "1999",
         address = "London",
         edition = "Second",
         isbn = "034069243X",
         pointers = "https://permalink.obvsg.at/AC02927008")

bibentry(bibtype = "Book",
         key = "R:Tufte:1990",
         author = person(given = c("Edward", "R."),
                         family = "Tufte"),
         title = "The Visual Display of Quantitative Information",
         publisher = "Graphics Press",
         year = "1990",
         address = "Cheshire, CT",
         isbn = "978-0961392109",
         pointers = "https://permalink.obvsg.at/AC00159112")

bibentry(bibtype = "TechReport",
         key = "R:Joyner+Boore+Porcella:1981",
         author = c(person(given = c("William", "B."),
                           family = "Joyner"),
                    person(given = c("David", "M."),
                           family = "Boore"),
                    person(given = c("Ronald", "L."),
                           family = "Porcella")),
         title = "Peak Horizontal Acceleration and Velocity from Strong-Motion Records Including Records from the 1979 {Imperial Valley}, {California} Earthquake",
         institution = "United States Department of the Interior",
         year = "1981",
         number = "USGS Open-File Repot 81-365",
         address = "Menlo Park, CA",
         url = "https://pubs.usgs.gov/of/1981/0365/")

bibentry(bibtype = "Book",
         key = "R:Chatterjee+Price:1977",
         author = c(person(given = "Samprit",
                           family = "Chatterjee"),
                    person(given = "Bertram",
                           family = "Price")),
         title = "Regression Analysis by Example",
         publisher = "Wiley",
         year = "1977",
         address = "New York, NY",
         isbn = "0471015210",
         pointers = "https://permalink.obvsg.at/AC00166496")

bibentry(bibtype = "Book",
         key = "R:Chatterjee+Price:1991",
         author = c(person(given = "Samprit",
                           family = "Chatterjee"),
                    person(given = "Bertram",
                           family = "Price")),
         title = "Regression Analysis by Example",
         publisher = "Wiley",
         year = "1991",
         address = "New York, NY",
         edition = "Second",
         isbn = "0471884790",
         pointers = "https://permalink.obvsg.at/AC00603591")

bibentry(bibtype = "Book",
         key = "R:Ezekiel:1930",
         author = person(given = "Mordecai",
                         family = "Ezekiel"),
         title = "Methods of Correlation Analysis",
         publisher = "Wiley",
         year = "1930",
         address = "New York, NY")

bibentry(bibtype = "Book",
         key = "R:Breslow+Day:1980",
         author = c(person(given = c("Norman", "E."),
                           family = "Breslow"),
                    person(given = c("N.", "E."),
                           family = "Day")),
         title = "Statistical Methods in Cancer Research. Volume 1: The Analysis of Case-Control Studies",
         publisher = "IARC",
         year = "1980",
         address = "Lyon",
         isbn = "9283211324",
         pointers = "https://permalink.obvsg.at/AC00946682")

bibentry(bibtype = "TechReport",
         key = "R:Freeny:1977",
         author = person(given = c("Anne", "E."),
                         family = "Freeny"),
         title = "A Portable Linear Regression Package with Test Programs",
         institution = "Bell Laboratories",
         year = "1977",
         type = "Memorandum")

bibentry(bibtype = "Book",
         key = "R:Weekes:1986",
         author = person(given = c("A.", "J."),
                         family = "Weekes"),
         title = "A {Genstat} Primer",
         publisher = "Hodder Arnold",
         year = "1986",
         isbn = "978-0713136074")

bibentry(bibtype = "Book",
         key = "R:Anderson:1979",
         author = person(given = c("Oliver", "D."),
                         family = "Anderson"),
         title = "Time Series Analysis and Forecasting: The {Box-Jenkins} Approach",
         publisher = "Butterworths",
         year = "1979",
         address = "London",
         isbn = "040870926X",
         pointers = "https://permalink.obvsg.at/AC02404917")

bibentry(bibtype = "Book",
         key = "R:Weast:1973",
         editor = person(given = c("Robert", "C."),
                         family = "Weast"),
         title = "Handbook of Chemistry and Physics: A Ready-Reference Book of Chemical and Physical Data",
         publisher = "CRC Press",
         year = "1973",
         address = "Cleveland, OH",
         isbn = "9780878194544",
         pointers = "https://permalink.obvsg.at/AC11011322")

bibentry(bibtype = "Misc",
         key = "R:USDOC:1977:a",
         author = person(given = "U.S. Department of Commerce, Bureau of the Census"),
         title = "Statistical Abstract of the United States",
         year = "1977")

bibentry(bibtype = "Misc",
         key = "R:USDOC:1977:b",
         author = person(given = "U.S. Department of Commerce, Bureau of the Census"),
         title = "County and City Data Book",
         year = "1977")

bibentry(bibtype = "Book",
         key = "R:Tong:1996",
         author = person(given = "Howell",
                         family = "Tong"),
         title = "Non-Linear Time Series",
         publisher = "Clarendon Press",
         year = "1996",
         address = "Oxford",
         isbn = "0198523009",
         pointers = "https://permalink.obvsg.at/AC01648164")

bibentry(bibtype = "Book",
         key = "R:Andrews+Herzberg:1985",
         author = c(person(given = c("David", "F."),
                           family = "Andrews"),
                    person(given = c("Agnes", "Margaret"),
                           family = "Herzberg")),
         title = "Data: A Collection of Problems from Many Fields for the Student and Research Worker",
         publisher = "Springer",
         year = "1985",
         address = "New York, NY",
         isbn = "9781461250982",
         doi = "10.1007/978-1-4612-5098-2",
         pointers = "https://permalink.obvsg.at/AC11899563")

bibentry(bibtype = "Book",
         key = "R:Mosteller+Tukey:1977",
         author = c(person(given = "Frederick",
                           family = "Mosteller"),
                    person(given = c("John", "Wilder"),
                           family = "Tukey")),
         title = "Data Analysis and Regression: A Second Course in Statistics",
         publisher = "Addison-Wesley",
         year = "1977",
         address = "Reading, MA",
         isbn = "020104854X",
         pointers = "https://permalink.obvsg.at/AC00231525")

bibentry(bibtype = "Book",
         key = "R:Meyer:1953",
         author = person(given = c("Hans", "Arthur"),
                         family = "Meyer"),
         title = "Forest Mensuration",
         publisher = "Penns Valley Publishers",
         year = "1953",
         pointers = "https://permalink.obvsg.at/AC03808292")

bibentry(bibtype = "Book",
         key = "R:Ryan+Joiner+Ryan:1976",
         author = c(person(given = c("Thomas", "A."),
                           family = "Ryan"),
                    person(given = c("Brian", "L."),
                           family = "Joiner"),
                    person(given = c("Barbara", "F."),
                           family = "Ryan")),
         title = "The {Minitab} Student Handbook",
         publisher = "Duxbury Press",
         year = "1976",
         address = "North Scituate, MA",
         isbn = "087150359X",
         pointers = "https://permalink.obvsg.at/AC03095277")

bibentry(bibtype = "Book",
         key = "R:Tippett:1950",
         author = person(given = c("Leonard", "H.", "C."),
                         family = "Tippett"),
         title = "Technological Applications of Statistics",
         publisher = "Wiley",
         year = "1950",
         address = "New York, NY",
         pointers = "https://permalink.obvsg.at/AC01762178")

bibentry(bibtype = "Book",
         key = "R:Crystal:1990",
         editor = person(given = "David",
                         family = "Crystal"),
         title = "The {Cambridge} Encyclopaedia",
         publisher = "Cambridge University Press",
         year = "1990",
         address = "Cambridge",
         isbn = "0521395283",
         pointers = "https://permalink.obvsg.at/AC00144062")
