



R -h
IF %ERRORLEVEL% NEQ 0 exit /B 1
R --version
IF %ERRORLEVEL% NEQ 0 exit /B 1
R -e \"library('tcltk')\""
IF %ERRORLEVEL% NEQ 0 exit /B 1
Rscript --version
IF %ERRORLEVEL% NEQ 0 exit /B 1
Rscript -e  'cat("ok\\n")'
IF %ERRORLEVEL% NEQ 0 exit /B 1
R -e \"capabilities()\""
IF %ERRORLEVEL% NEQ 0 exit /B 1
R -e \"grSoftVersion()\""
IF %ERRORLEVEL% NEQ 0 exit /B 1
open
IF %ERRORLEVEL% NEQ 0 exit /B 1
Rfe --help
IF %ERRORLEVEL% NEQ 0 exit /B 1
Rterm --help
IF %ERRORLEVEL% NEQ 0 exit /B 1
Rterm --version
IF %ERRORLEVEL% NEQ 0 exit /B 1
Rscript -e "stopifnot(capabilities('jpeg'), TRUE)"
IF %ERRORLEVEL% NEQ 0 exit /B 1
Rscript -e "stopifnot(capabilities('png'), TRUE)"
IF %ERRORLEVEL% NEQ 0 exit /B 1
if not exist %PREFIX%\\Lib\\R\\bin\\x64\\R.lib exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
