Test file for custom graphics devices
<<results=hide>>=
my.Swd <- function(name, width, height, ...) {
  cat("running my.Swd\n")
  grDevices::png(filename = paste(name, "png", sep = "."),
                 width = width, height = height, res = 100, units = "in",
                 bg = "transparent")
}
my.Swd.off <- function() {
    cat("shutting down my.Swd\n")
    grDevices::dev.off()
}
<<fig=TRUE, grdevice=my.Swd>>=
plot(1:2)
<<fig=TRUE, pdf=TRUE, grdevice=my.Swd>>=
plot(1:3)
<<fig=TRUE, grdevice=my.Swd, pdf=TRUE>>=
plot(1:4)
@
Done!
