\SweaveOpts{keep.source=TRUE}

Now a chunk starting with a comment, lost in R 2.12.0 patched:
<<ex>>=
## This is an initial comment line. Let's hope it's not being lost
"Above there's a comment."
1:10 # a comment here is preserved
pi   # also, there's one the next (last) line - which used to get lost
## and a last comment here ... lost also in earlier R versions
@

and some more text.
